@@ -1,4 +1,4 @@
-from __future__ import annotations  # todo0 remove in 3.11
+from __future__ import annotations  # todo0 remove when it's by default
 
 __all__ = [
     'find_message',
@@ -17,6 +17,7 @@ __all__ = [
 import datetime
 import functools
 import random
+import traceback
 from abc import ABC, abstractmethod
 from typing import Any, Callable, Generic, Iterable, Type, TypeVar, overload
 
@@ -27,8 +28,8 @@ import telethon.events.common
 from flanautils import AmbiguityError, Media, NotFoundError, OrderedSet, RatioMatch, return_if_first_empty, shift_args_if_called
 
 from multibot import constants
-from multibot.exceptions import LimitError, SendError
-from multibot.models import Ban, BotAction, Button, ButtonsGroup, Chat, Message, Mute, Platform, RegisteredButtonCallback, RegisteredCallback, Role, User
+from multibot.exceptions import BadRoleError, LimitError, SendError, UserDisconnectedError
+from multibot.models import Ban, Button, Chat, Message, Mute, Penalty, Platform, RegisteredButtonCallback, RegisteredCallback, RegisteredCallbackBase, Role, User
 
 
 # ---------------------------------------------------------- #
@@ -143,7 +144,7 @@ def out_of_service(func: Callable) -> Callable:
     @functools.wraps(func)
     @find_message
     async def wrapper(self: MultiBot, message: Message):
-        if self.is_bot_mentioned(message) or message.chat.is_private:
+        if message.chat.is_private or self.is_bot_mentioned(message):
             await self.send(random.choice(constants.OUT_OF_SERVICES_PHRASES), message)
         await self.accept_button_event(message)
 
@@ -196,11 +197,11 @@ def parse_arguments(func: Callable) -> Callable:
                     text = str(number)
                 case Media() as media:
                     pass
-                case User() as user:
+                case self.User() as user:
                     chat = user
-                case Chat() as chat:
+                case self.Chat() as chat:
                     pass
-                case Message() as message:
+                case self.Message() as message:
                     pass
                 case _:
                     buttons = parse_buttons(arg)
@@ -260,12 +261,11 @@ T = TypeVar('T')
 
 class MultiBot(Generic[T], ABC):
     client: T
+    Chat = Chat
+    Message = Message
+    User = User
 
     def __init__(self, bot_token: str, bot_client: T):
-        self.Chat = Chat
-        self.Message = Message
-        self.User = User
-
         self.platform: Platform | None = None
         self.id: int | None = None
         self.name: str | None = None
@@ -281,34 +281,32 @@ class MultiBot(Generic[T], ABC):
     # -------------------- PROTECTED METHODS -------------------- #
     # ----------------------------------------------------------- #
     def _add_handlers(self):
-        self.register(self._on_ban, constants.KEYWORDS['ban'])
-
-        self.register(self._on_delete, constants.KEYWORDS['delete'])
-        self.register(self._on_delete, (constants.KEYWORDS['delete'], constants.KEYWORDS['message']))
-
-        self.register(self._on_mute, constants.KEYWORDS['mute'])
-        self.register(self._on_mute, (('haz', 'se'), constants.KEYWORDS['mute']))
-        self.register(self._on_mute, (constants.KEYWORDS['deactivate'], constants.KEYWORDS['unmute']))
-        self.register(self._on_mute, (constants.KEYWORDS['deactivate'], constants.KEYWORDS['sound']))
-
-        self.register(self._on_unban, constants.KEYWORDS['unban'])
-
-        self.register(self._on_unmute, constants.KEYWORDS['unmute'])
-        self.register(self._on_unmute, (constants.KEYWORDS['deactivate'], constants.KEYWORDS['mute']))
-        self.register(self._on_unmute, (constants.KEYWORDS['activate'], constants.KEYWORDS['sound']))
-
-        self.register(self._on_users, constants.KEYWORDS['user'])
-
-        self.register_button(self._on_users_button_press, ButtonsGroup.USERS)
+        pass
 
     async def _ban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
-    @staticmethod
-    async def _check_messages():
-        before_date = datetime.datetime.now(datetime.timezone.utc) - constants.MESSAGE_EXPIRATION_TIME
-        Message.collection.delete_many({'date': {'$lte': before_date}})
-        BotAction.collection.delete_many({'date': {'$lte': before_date}})
+    async def _call_registered_callback(self, callback: RegisteredCallbackBase, message: Message):
+        try:
+            await callback(message)
+        except Exception:
+            if constants.SEND_EXCEPTION_MESSAGE_LINES:
+                await self.send(f'{random.choice(constants.EXCEPTION_PHRASES)}\n'
+                                '\n'
+                                '...\n'
+                                '\n'.join(traceback.format_exc().splitlines()[-constants.SEND_EXCEPTION_MESSAGE_LINES:]),
+                                message)
+            raise
+
+    async def _check_penalties(self, penalty_class: Type[Penalty], unpenalize_method: Callable):
+        penalties = penalty_class.find({'platform': self.platform.value}, lazy=True)
+
+        for penalty in penalties:
+            if penalty.until and penalty.until <= datetime.datetime.now(datetime.timezone.utc):
+                try:
+                    await self._remove_penalty(penalty, unpenalize_method)
+                except UserDisconnectedError:
+                    pass
 
     async def _find_users_to_punish(self, message: Message) -> OrderedSet[User]:
         bot_user = await self.get_me(message.chat.group_id)
@@ -353,7 +351,7 @@ class MultiBot(Generic[T], ABC):
     async def _get_message(self, event: constants.MESSAGE_EVENT) -> Message:
         original_message = event if isinstance(event, constants.ORIGINAL_MESSAGE) else await self._get_original_message(event)
 
-        message = Message(
+        message = self.Message(
             platform=self.platform,
             id=await self._get_message_id(original_message),
             author=await self._get_author(original_message),
@@ -409,6 +407,8 @@ class MultiBot(Generic[T], ABC):
                 await self.send_error(str(e), context)
             except (SendError, NotFoundError) as e:
                 await self.send_error(str(e), context)
+            except UserDisconnectedError:
+                await self.send_error('El usuario no está conectado.', context)
             except AmbiguityError:
                 if constants.RAISE_AMBIGUITY_ERROR:
                     await self.send_error(f'Hay varias acciones relacionadas con tu mensaje. ¿Puedes especificar un poco más? {random.choice(constants.SAD_EMOJIS)}', context)
@@ -425,15 +425,20 @@ class MultiBot(Generic[T], ABC):
         minimum_ratio_to_match: float = constants.MINIMUM_RATIO_TO_MATCH
     ) -> OrderedSet[RegisteredCallback]:
         text = text.lower()
-        text = flanautils.replace(text, {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u',
-                                         'à': 'a', 'è': 'e', 'ì': 'i', 'ò': 'o', 'ù': 'u',
-                                         'â': 'a', 'ê': 'e', 'î': 'i', 'ô': 'o', 'û': 'u',
-                                         'ä': 'a', 'ë': 'e', 'ï': 'i', 'ö': 'o', 'ü': 'u'})
-        text = flanautils.translate(text, {'?': ' ', '¿': ' ', '!': ' ', '¡': ' '})
-        original_text_words = OrderedSet(text.split())
+        original_text_words = OrderedSet()
+        for word in text.split():
+            if len(word) > constants.MAX_WORD_LENGTH:
+                continue
+
+            word = flanautils.replace(word, {'á': 'a', 'é': 'e', 'í': 'i', 'ó': 'o', 'ú': 'u',
+                                             'à': 'a', 'è': 'e', 'ì': 'i', 'ò': 'o', 'ù': 'u',
+                                             'â': 'a', 'ê': 'e', 'î': 'i', 'ô': 'o', 'û': 'u',
+                                             'ä': 'a', 'ë': 'e', 'ï': 'i', 'ö': 'o', 'ü': 'u'})
+            word = flanautils.translate(word, {'?': ' ', '¿': ' ', '!': ' ', '¡': ' '})
+            original_text_words.add(word)
         text_words = original_text_words - flanautils.CommonWords.get()
 
-        matched_callbacks: set[RatioMatch[RegisteredCallback]] = set()
+        matched_callbacks: set[tuple[int, RatioMatch[RegisteredCallback]]] = set()
         always_callbacks: set[RegisteredCallback] = set()
         default_callbacks: set[RegisteredCallback] = set()
         for registered_callback in registered_callbacks:
@@ -457,20 +462,22 @@ class MultiBot(Generic[T], ABC):
                         mached_keywords_groups += 1
 
                 if mached_keywords_groups and mached_keywords_groups == len(registered_callback.keywords):
-                    for matched_callback in matched_callbacks:  # If the callback has been matched before but with less score it is overwritten, otherwise it is added
+                    for priority, matched_callback in matched_callbacks:  # If the callback has been matched before but with less score it is overwritten, otherwise it is added
                         if matched_callback.element.callback == registered_callback.callback:
                             if total_ratio > matched_callback.ratio:
-                                matched_callbacks.discard(matched_callback)
-                                matched_callbacks.add(RatioMatch(registered_callback, total_ratio))
+                                matched_callbacks.discard((priority, matched_callback))
+                                matched_callbacks.add((registered_callback.priority, RatioMatch(registered_callback, total_ratio)))
                             break
                     else:
-                        matched_callbacks.add(RatioMatch(registered_callback, total_ratio))
+                        matched_callbacks.add((registered_callback.priority, RatioMatch(registered_callback, total_ratio)))
 
-        match sorted(matched_callbacks):
-            case [single]:
+        sorted_matched_callbacks = sorted(matched_callbacks, key=lambda e: e[1])
+        sorted_matched_callbacks = sorted(sorted_matched_callbacks, key=lambda e: e[0], reverse=True)
+        match sorted_matched_callbacks:
+            case [(_priority, single)]:
                 determined_callbacks = always_callbacks | {single.element}
-            case [first, second, *_] if first.ratio >= minimum_ratio_to_match:
-                if first.ratio == second.ratio:
+            case [(first_priority, first), (second_priority, second), *_] if first.ratio >= minimum_ratio_to_match:
+                if first_priority == second_priority and first.ratio == second.ratio:
                     raise AmbiguityError(f'\n{first.element.callback}\n{second.element.callback}')
                 determined_callbacks = always_callbacks | {first.element}
             case _:
@@ -478,22 +485,32 @@ class MultiBot(Generic[T], ABC):
 
         return OrderedSet(registered_callback for registered_callback in registered_callbacks if registered_callback in determined_callbacks)
 
+    async def _remove_penalty(self, penalty: Penalty, unpenalize_method: Callable, message: Message = None, delete=True):
+        try:
+            await unpenalize_method(penalty.user_id, penalty.group_id)
+        except (BadRoleError, UserDisconnectedError) as e:
+            if message and message.chat.original_object:
+                await self._manage_exceptions(e, message)
+            else:
+                raise e
+        else:
+            if delete:
+                penalty.pull_from_database()
+                penalty.delete()
+
     async def _unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
     async def _unmute(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         pass
 
+    async def _unpenalize_later(self, penalty: Penalty, unpenalize_method: Callable, message: Message = None):
+        if penalty.time and datetime.timedelta() <= penalty.time <= constants.TIME_THRESHOLD_TO_MANUAL_UNPUNISH:
+            await flanautils.do_later(penalty.time, self._remove_penalty, penalty, unpenalize_method, message)
+
     # ---------------------------------------------- #
     #                    HANDLERS                    #
     # ---------------------------------------------- #
-    @bot_mentioned
-    @group
-    @admin(send_negative=True)
-    async def _on_ban(self, message: Message):
-        for user in await self._find_users_to_punish(message):
-            await self.ban(user, message, flanautils.words_to_time(message.text), message)
-
     @find_message
     async def _on_button_press_raw(self, message: Message):
         if getattr(message.buttons_info, 'key', None) is None:
@@ -501,32 +518,7 @@ class MultiBot(Generic[T], ABC):
 
         for registered_callback in self._registered_button_callbacks:
             if registered_callback.key == message.buttons_info.key:
-                await registered_callback(message)
-
-    @inline(False)
-    async def _on_delete(self, message: Message):
-        if message.replied_message:
-            if message.replied_message.author.id == self.id:
-                await self.delete_message(message.replied_message)
-                await self.delete_message(message)
-            elif self.is_bot_mentioned(message):
-                await self.send_negative(message)
-        elif message.author.is_admin and self.is_bot_mentioned(message) and (n_messages := flanautils.sum_numbers_in_text(message.text)):
-            if n_messages <= 0:
-                await self._manage_exceptions(ValueError(), message)
-                return
-
-            try:
-                await self.clear(n_messages, message.chat)
-            except LimitError as e:
-                await self._manage_exceptions(e, message)
-
-    @group
-    @bot_mentioned
-    @admin(send_negative=True)
-    async def _on_mute(self, message: Message):
-        for user in await self._find_users_to_punish(message):
-            await self.mute(user, message, flanautils.words_to_time(message.text), message)
+                await self._call_registered_callback(registered_callback, message)
 
     @ignore_self_message
     async def _on_new_message_raw(self, message: Message):
@@ -536,67 +528,14 @@ class MultiBot(Generic[T], ABC):
             await self._manage_exceptions(e, message)
         else:
             for registered_callback in registered_callbacks:
-                await registered_callback(message)
+                await self._call_registered_callback(registered_callback, message)
 
     async def _on_ready(self):
+        flanautils.init_db()
         print(f'{self.name} activado en {self.platform.name} (id: {self.id})')
-        await flanautils.do_every(constants.CHECK_MESSAGE_EVERY_SECONDS, self._check_messages)
-        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, Mute.check_olds, self._unmute, self.platform)
-        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, Ban.check_olds, self._unban, self.platform)
-
-    @bot_mentioned
-    @group
-    @admin(send_negative=True)
-    async def _on_unban(self, message: Message):
-        for user in await self._find_users_to_punish(message):
-            await self.unban(user, message, message)
-
-    @group
-    @bot_mentioned
-    @admin(send_negative=True)
-    async def _on_unmute(self, message: Message):
-        for user in await self._find_users_to_punish(message):
-            await self.unmute(user, message, message)
-
-    @group
-    @bot_mentioned
-    async def _on_users(self, message: Message):
-        role_names = [role.name for role in await self.get_group_roles(message.chat.group_id)]
-        role_names.remove('@everyone')
-
-        user_names = [f'<@{user.id}>' for user in await self.find_users_by_roles([], message)]
-        joined_user_names = ', '.join(user_names)
-        await self.delete_message(message)
-        bot_message = await self.send(
-            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:</b>",
-            flanautils.chunks([f'❌ {role_name}' for role_name in role_names], 5),
-            message,
-            buttons_key=ButtonsGroup.USERS
-        )
-        await self.edit(f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:</b>\n{joined_user_names}\n\n<b>Filtrar usuarios por roles:</b>", bot_message)
-
-    async def _on_users_button_press(self, message: Message):
-        await self.accept_button_event(message)
-
-        try:
-            button_role_name = message.buttons_info.pressed_text.split(maxsplit=1)[1]
-        except IndexError:
-            return
-
-        pressed_button = message.buttons_info.pressed_button
-        pressed_button.is_checked = not pressed_button.is_checked
-        pressed_button.text = f"{'✔' if pressed_button.is_checked else '❌'} {button_role_name}"
-
-        selected_role_names = [checked_button.text.split(maxsplit=1)[1] for checked_button in message.buttons_info.checked_buttons]
-        user_names = [f'<@{user.id}>' for user in await self.find_users_by_roles(selected_role_names, message)]
-        joined_user_names = ', '.join(user_names)
-        await self.edit(
-            f"<b>{len(user_names)} usuario{'' if len(user_names) == 1 else 's'}:</b>\n"
-            f"{joined_user_names}\n\n"
-            f"<b>Filtrar usuarios por roles:</b>",
-            message.buttons_info.buttons,
-            message
-        )
+        await flanautils.do_every(constants.CHECK_MESSAGE_EVERY_SECONDS, self.clear_old_database_items)
+        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_bans)
+        await flanautils.do_every(constants.CHECK_MUTES_EVERY_SECONDS, self.check_mutes)
 
     # -------------------------------------------------------- #
     # -------------------- PUBLIC METHODS -------------------- #
@@ -610,12 +549,25 @@ class MultiBot(Generic[T], ABC):
     async def ban(self, user: int | str | User, group_: int | str | Chat | Message, time: int | datetime.timedelta = None, message: Message = None):
         # noinspection PyTypeChecker
         ban = Ban(self.platform, self.get_user_id(user), self.get_group_id(group_), time)
-        await ban.punish(self._ban, self._unban, message)
+        await self._ban(ban.user_id, ban.group_id, message)
+        ban.save(pull_exclude_fields=('until',))
+        await self._unpenalize_later(ban, self._unban, message)
+
+    async def check_bans(self):
+        await self._check_penalties(Ban, self._unban)
+
+    async def check_mutes(self):
+        await self._check_penalties(Mute, self._unmute)
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def clear(self, n_messages: int, chat: int | str | Chat | Message):
         pass
 
+    @classmethod
+    async def clear_old_database_items(cls):
+        before_date = datetime.datetime.now(datetime.timezone.utc) - constants.MESSAGE_EXPIRATION_TIME
+        cls.Message.collection.delete_many({'date': {'$lte': before_date}})
+
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
     async def delete_message(self, message_to_delete: int | str | Message, chat: int | str | Chat | Message = None):
         pass
@@ -656,12 +608,12 @@ class MultiBot(Generic[T], ABC):
                 return group_id
             case str(group_name):
                 try:
-                    return Chat.find_one({'platform': self.platform.value, 'group_name': group_name}).group_id
+                    return self.Chat.find_one({'platform': self.platform.value, 'group_name': group_name}).group_id
                 except AttributeError:
                     return
-            case Chat() as chat:
+            case self.Chat() as chat:
                 return chat.group_id
-            case Message() as message:
+            case self.Message() as message:
                 return message.chat.group_id
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
@@ -669,14 +621,14 @@ class MultiBot(Generic[T], ABC):
         match group_:
             case int(group_id):
                 try:
-                    return Chat.find_one({'platform': self.platform.value, 'group_id': group_id}).group_name
+                    return self.Chat.find_one({'platform': self.platform.value, 'group_id': group_id}).group_name
                 except AttributeError:
                     return
             case str(group_name):
                 return group_name
-            case Chat() as chat:
+            case self.Chat() as chat:
                 return chat.group_name
-            case Message() as message:
+            case self.Message() as message:
                 return message.chat.group_name
 
     async def get_me(self, group_: int | str | Chat | Message = None) -> User | None:
@@ -706,10 +658,10 @@ class MultiBot(Generic[T], ABC):
                 return user_id
             case str(user_name):
                 try:
-                    return User.find_one({'platform': self.platform.value, 'name': user_name}).id
+                    return self.User.find_one({'platform': self.platform.value, 'name': user_name}).id
                 except AttributeError:
                     return
-            case User():
+            case self.User():
                 return user.id
 
     @return_if_first_empty(exclude_self_types='MultiBot', globals_=globals())
@@ -717,12 +669,12 @@ class MultiBot(Generic[T], ABC):
         match user:
             case int(user_id):
                 try:
-                    return User.find_one({'platform': self.platform.value, 'id': user_id}).name
+                    return self.User.find_one({'platform': self.platform.value, 'id': user_id}).name
                 except AttributeError:
                     return
             case str(user_name):
                 return user_name
-            case User():
+            case self.User():
                 return user.name
 
     def is_bot_mentioned(self, message: Message) -> bool:
@@ -740,23 +692,32 @@ class MultiBot(Generic[T], ABC):
     async def is_self_muted(self, user: int | str | User, group_: int | str | Chat | Message) -> bool:
         pass
 
-    async def mute(self, user: int | str | User, group_: int | str | Chat | Message, time: int | datetime.timedelta, message: Message = None):
+    async def mute(self, user: int | str | User, group_: int | str | Chat | Message, time: int | datetime.timedelta = None, message: Message = None):
         # noinspection PyTypeChecker
         mute = Mute(self.platform, self.get_user_id(user), self.get_group_id(group_), time)
-        await mute.punish(self._mute, self._unmute, message)
+        try:
+            await self._mute(mute.user_id, mute.group_id)
+        except UserDisconnectedError as e:
+            if message and message.chat.original_object:
+                await self._manage_exceptions(e, message)
+            else:
+                raise e
+        else:
+            mute.save(pull_exclude_fields=('until',))
+            await self._unpenalize_later(mute, self._unmute, message)
 
     @overload
-    def register(self, func_: Callable = None, keywords=(), min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords=(), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
         pass
 
     @overload
-    def register(self, keywords=(), min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, keywords=(), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
         pass
 
     @shift_args_if_called(exclude_self_types='MultiBot', globals_=globals())
-    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
+    def register(self, func_: Callable = None, keywords: str | Iterable[str | Iterable[str]] = (), priority: int | float = 1, min_ratio=constants.PARSE_CALLBACKS_MIN_RATIO_DEFAULT, always=False, default=False):
         def decorator(func):
-            self._registered_callbacks.append(RegisteredCallback(func, keywords, min_ratio, always, default))
+            self._registered_callbacks.append(RegisteredCallback(func, keywords, priority, min_ratio, always, default))
             return func
 
         return decorator(func_) if func_ else decorator
@@ -800,9 +761,9 @@ class MultiBot(Generic[T], ABC):
         pass
 
     @parse_arguments
-    async def send_error(self, *args, exceptions_to_ignore: Type[BaseException] | Iterable[Type[BaseException]] = (), **kwargs) -> constants.ORIGINAL_MESSAGE:
+    async def send_error(self, *args, exceptions_to_capture: Type[BaseException] | Iterable[Type[BaseException]] = (), **kwargs) -> constants.ORIGINAL_MESSAGE:
         bot_message = await self.send(*args, **kwargs)
-        await flanautils.do_later(constants.ERROR_MESSAGE_DURATION, self.delete_message, bot_message, exceptions_to_ignore=exceptions_to_ignore or discord.errors.NotFound)
+        await flanautils.do_later(constants.ERROR_MESSAGE_DURATION, self.delete_message, bot_message, exceptions_to_capture=exceptions_to_capture or discord.errors.NotFound)
         return bot_message
 
     @inline
@@ -825,9 +786,9 @@ class MultiBot(Generic[T], ABC):
     async def unban(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         # noinspection PyTypeChecker
         ban = Ban(self.platform, self.get_user_id(user), self.get_group_id(group_))
-        await ban.unpunish(self._unban, message)
+        await self._remove_penalty(ban, self._unban, message)
 
     async def unmute(self, user: int | str | User, group_: int | str | Chat | Message, message: Message = None):
         # noinspection PyTypeChecker
         mute = Mute(self.platform, self.get_user_id(user), self.get_group_id(group_))
-        await mute.unpunish(self._unmute, message)
+        await self._remove_penalty(mute, self._unmute, message)
