@@ -1,4 +1,4 @@
-from __future__ import annotations  # todo0 remove in 3.11
+from __future__ import annotations  # todo0 remove when it's by default
 
 __all__ = ['user_client', 'TelegramBot']
 
@@ -9,6 +9,7 @@ import pathlib
 from typing import Any, Callable
 
 import flanautils
+import pymongo
 import telethon.events.common
 import telethon.hints
 import telethon.tl.functions.channels
@@ -49,6 +50,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         self.phone = phone
         self.bot_session = bot_session
         self.user_session = user_session
+        self.inline_call_index = 0
 
         if self.bot_session:
             bot_client = TelegramClient(StringSession(bot_session), self.api_id, self.api_hash)
@@ -102,7 +104,7 @@ class TelegramBot(MultiBot[TelegramClient]):
             group_id = original_chat.id
             group_name = chat_name
 
-        return Chat(
+        return self.Chat(
             platform=self.platform,
             id=original_chat.id,
             name=chat_name,
@@ -119,7 +121,7 @@ class TelegramBot(MultiBot[TelegramClient]):
         except (AttributeError, TypeError, ValueError):
             is_admin = None
 
-        return User(
+        return self.User(
             platform=self.platform,
             id=original_user.id,
             name=self._get_name_from_entity(original_user),
@@ -243,7 +245,13 @@ class TelegramBot(MultiBot[TelegramClient]):
     # ---------------------------------------------- #
     @find_message
     async def _on_inline_query_raw(self, message: Message):
-        await super()._on_new_message_raw(message)
+        self.inline_call_index += 1
+        inline_call_index = self.inline_call_index
+        await asyncio.sleep(constants.INLINE_DELAY_SECONDS)
+
+        if inline_call_index == self.inline_call_index:
+            self.inline_call_index = 0
+            await self._on_new_message_raw(message)
 
     async def _on_ready(self):
         self.platform = Platform.TELEGRAM
@@ -260,7 +268,7 @@ class TelegramBot(MultiBot[TelegramClient]):
     # -------------------------------------------------------- #
     async def accept_button_event(self, event: constants.TELEGRAM_EVENT | Message):
         match event:
-            case Message():
+            case self.Message():
                 event = event.original_event
 
         try:
@@ -305,23 +313,24 @@ class TelegramBot(MultiBot[TelegramClient]):
 
         async with self.user_client:
             owner_user = await self._create_user_from_telegram_user(await self.user_client.get_me(), chat.group_id)
-            if owner_user not in chat.users:
+            if owner_user not in await self.get_users(chat):
                 return
 
-            user_chat = await self.user_client.get_entity(chat.id)
-            messages_to_delete = await self.user_client.get_messages(user_chat, n_messages)
-            await self.user_client.delete_messages(user_chat, messages_to_delete)
-            for message_to_delete in messages_to_delete:
-                message_to_delete.is_deleted = True
-                message_to_delete.save()
+            original_chat = await self.user_client.get_entity(chat.id)  # it's necessary
+            messages_to_delete = await self.user_client.get_messages(original_chat, n_messages)
+            await self.user_client.delete_messages(original_chat, messages_to_delete)
+            database_messages_to_delete_generator = self.Message.find({'platform': self.platform.value, 'chat': chat.object_id}, sort_keys=(('date', pymongo.DESCENDING),), limit=n_messages, lazy=True)
+            for database_message_to_delete in database_messages_to_delete_generator:
+                database_message_to_delete.is_deleted = True
+                database_message_to_delete.save()
 
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def delete_message(self, message_to_delete: int | str | Message, chat: int | str | Chat | Message = None):
         chat = await self.get_chat(chat)
         match message_to_delete:
             case int() | str():
-                message_to_delete = Message.find_one({'platform': self.platform.value, 'id': str(message_to_delete), 'chat': chat.object_id})
-            case Message() if message_to_delete.original_object and message_to_delete.chat and message_to_delete.chat == chat:
+                message_to_delete = self.Message.find_one({'platform': self.platform.value, 'id': str(message_to_delete), 'chat': chat.object_id})
+            case self.Message() if message_to_delete.original_object and message_to_delete.chat and message_to_delete.chat == chat:
                 chat = None
 
         if chat and chat.original_object:
@@ -337,13 +346,13 @@ class TelegramBot(MultiBot[TelegramClient]):
     @return_if_first_empty(exclude_self_types='TelegramBot', globals_=globals())
     async def get_chat(self, chat: int | str | User | Chat | Message) -> Chat | None:
         match chat:
-            case User() as user:
+            case self.User() as user:
                 if user.original_object:
                     return await self._create_chat_from_telegram_chat(user.original_object)
                 chat_id = user.id
-            case Chat():
+            case self.Chat():
                 return chat
-            case Message() as message:
+            case self.Message() as message:
                 return message.chat
             case _ as chat_id:
                 pass
@@ -360,7 +369,7 @@ class TelegramBot(MultiBot[TelegramClient]):
                 pass
             case str(message_id):
                 pass
-            case Message():
+            case self.Message():
                 return message
             case _:
                 raise TypeError('bad arguments')
@@ -456,20 +465,29 @@ class TelegramBot(MultiBot[TelegramClient]):
         match reply_to:
             case str():
                 reply_to = int(reply_to)
-            case Message() as message_to_reply:
+            case self.Message() as message_to_reply:
                 reply_to = message_to_reply.original_object
 
         with flanautils.suppress_stderr():
-            try:
-                original_message = await self.client.send_message(chat.original_object, text, buttons=telegram_buttons, reply_to=reply_to, silent=silent, **kwargs)
-            except telethon.errors.rpcerrorlist.WebpageCurlFailedError:
-                if media.bytes_:
-                    kwargs['file'] = await self._prepare_media_to_send(media, prefer_bytes=True)
+            parse_retries = 1
+            while True:
+                try:
                     original_message = await self.client.send_message(chat.original_object, text, buttons=telegram_buttons, reply_to=reply_to, silent=silent, **kwargs)
+                except telethon.errors.rpcerrorlist.WebpageCurlFailedError:
+                    if media.bytes_:
+                        kwargs['file'] = await self._prepare_media_to_send(media, prefer_bytes=True)
+                    else:
+                        raise
+                except ValueError as e:
+                    if 'parse' in str(e).lower() and parse_retries:
+                        del kwargs['parse_mode']
+                        parse_retries -= 1
+                    else:
+                        raise
+                except (telethon.errors.rpcerrorlist.PeerIdInvalidError, telethon.errors.rpcerrorlist.UserIsBlockedError):
+                    return
                 else:
-                    raise
-            except (telethon.errors.rpcerrorlist.PeerIdInvalidError, telethon.errors.rpcerrorlist.UserIsBlockedError):
-                return
+                    break
 
         return await self._create_bot_message_from_telegram_bot_message(original_message, media, chat, buttons, buttons_key, contents)
 
