@@ -1,6 +1,7 @@
 import datetime
 
 import discord.ext.commands
+import flanautils
 import telethon.events.common
 import twitchio
 
@@ -35,12 +36,15 @@ CHECK_MESSAGE_EVERY_SECONDS = datetime.timedelta(days=1).total_seconds()
 CHECK_MUTES_EVERY_SECONDS = datetime.timedelta(hours=1).total_seconds()
 COMMAND_MESSAGE_DURATION = 5
 DELETE_MESSAGE_LIMIT = 100
+DISCORD_BUTTON_MAX_CHARACTERS = 80
 DISCORD_BUTTONS_MAX = 5
-DISCORD_COMMAND_PREFIX = '/'
+DISCORD_COMMAND_PREFIX = flanautils.random_string()
 DISCORD_MEDIA_MAX_BYTES = 8000000
 ERROR_MESSAGE_DURATION = 10
+INLINE_DELAY_SECONDS = 2
 KEYWORDS_LENGHT_PENALTY = 0.001
 MAX_FILE_EXTENSION_LENGHT = 5
+MAX_WORD_LENGTH = 25
 MESSAGE_EXPIRATION_TIME = datetime.timedelta(weeks=1)
 MINIMUM_RATIO_TO_MATCH = 3
 PARSE_BUTTON_CALLBACKS_MIN_RATIO_DEFAULT = 1
@@ -48,17 +52,27 @@ PARSE_CALLBACKS_MIN_RATIO_DEFAULT = 0.93
 PYMONGO_MEDIA_MAX_BYTES = 15000000
 RAISE_AMBIGUITY_ERROR = False
 RATIO_REWARD_EXPONENT = 2
+SEND_EXCEPTION_MESSAGE_LINES = 5
 TELEGRAM_SEND_AS_FILE_RATIO_MIN_RATIO = 0.85
 TIME_THRESHOLD_TO_MANUAL_UNPUNISH = datetime.timedelta(days=3)
 
 SAD_EMOJIS = '😥😪😓😔😕☹🙁😞😢😭😩😰'
 
+EXCEPTION_PHRASES = ('A ver como lo digo...', 'Anda mira que error más bonito.', 'Ayudaaa',
+                     'Bueeeno... esto es incómodo...',
+                     'Con que querías que hiciera eso... pues esto es lo que he hecho:', 'Feliz Navidad:',
+                     'Flanagan, ayuda!', 'Ha pasado esto:', 'Han pasado cosas.', 'He hecho pum',
+                     'Iba a hacer eso peeero he hecho esto:', 'La cagué', 'La he cagao', 'Me fui a la puta',
+                     'Me hice caca', 'Me rompí', 'No funciono', 'No me siento muy bien...', 'Pues me he roto',
+                     'Pues no ha salido muy bien la cosa', 'Todo iba bien hasta que dejó de ir bien', 'Toma error',
+                     'me rompido')
+
 INTERROGATION_PHRASES = ('?', 'que?', 'que dise', 'no entiendo', 'no entender', 'mi no entender', 'ein?', '🤔', '🤨',
                          '🧐', '🙄', '🙃')
 
 KEYWORDS = {
     'activate': ('activa', 'activar', 'activate', 'add', 'añade', 'añadir', 'deja', 'dejale', 'devuelve', 'devuelvele',
-                 'enable', 'encender', 'enciende', 'habilita', 'habilitar'),
+                 'enable', 'encender', 'enciende', 'habilita', 'habilitar', 'permite'),
     'audio': ('audio', 'music', 'musica', 'sonido', 'sound'),
     'audit': ('audit', 'auditoria', 'desconectado', 'disconnect', 'log', 'move', 'mover', 'movido', 'registro'),
     'ban': ('ban', 'banea', 'banealo'),
